package org.arefgard.icerya.flow.integration;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;

import org.arefgard.icerya.flow.exception.DIContainerNotSupportedException;
import org.arefgard.icerya.flow.exception.UsecaseFlowException;
import org.arefgard.icerya.flow.integration.icerya.DIContainerIcerya;
import org.arefgard.icerya.flow.integration.seasar2.DIContainerSeasar2;
import org.arefgard.icerya.flow.integration.spring.DIContainerSpring;

/**
 * DIRei𐶐t@NgB
 * 
 * @author Takashi Yamashina
 * @since 1.0.0
 */
public class DIContainerFactory {
	
	private static final String CONTAINER_ICERYA = "Icerya";
	private static final String CONTAINER_SEASAR = "Seasar";
	private static final String CONTAINER_SPRING = "Spring";
	
	/**
	 * DIRei𐶐B
	 * <br/>
	 * <br/>
	 * DIRei̒`t@C̊gqdicon̏ꍇseasar2ṕA
	 * xml̏ꍇSpringṕAdix̏ꍇIceryaContainerpDI
	 * ReiIuWFNgԂB
	 * @param path DIReĩpX(NXpX)
	 * @return DIRei
	 * @throws UsecaseFlowException T|[gĂȂDIReiw肳ꂽꍇ
	 */
	public static DIContainer createContainer(String path) throws UsecaseFlowException {
		DIContainer container = null;
		if(getContainerType(path).equals(CONTAINER_SEASAR)) {
			container = new DIContainerSeasar2(path);
		}else if(getContainerType(path).equals(CONTAINER_SPRING)) {
			container = new DIContainerSpring(path);
		}else if(getContainerType(path).equals(CONTAINER_ICERYA)) {
			container = new DIContainerIcerya(path);
		}else {
			throw new DIContainerNotSupportedException();
		}
		return container;
	}
	
	private static String getContainerType(String path) throws UsecaseFlowException {
		InputStream is = null;
		ByteArrayOutputStream os = null;
		String head = null;
		String result = null;
		try {
			is = ClassLoader.getSystemResourceAsStream(path);
			if(is == null) {
				path.getClass().getClassLoader().getResourceAsStream(path);
			}
			os = new ByteArrayOutputStream();
			int len = 0;
			byte[] buf = new byte[4096];
			len = is.read(buf);
			os.write(buf, 0, len);
			head = os.toString();
		}catch(Exception e) {
			throw new DIContainerNotSupportedException(e);
		}finally {
			if(is != null) {
				try {
					is.close();
				}catch(Exception e) {
					// ignore
				}
			}
			if(os != null) {
				try {
					os.close();
				}catch(Exception e) {
					// ignore
				}
			}
		}
		// 擪͂
		if(head == null) {
			throw new DIContainerNotSupportedException();
		}else if(head.indexOf("<container") > 0) {
			// IceryaContainer
			result = CONTAINER_ICERYA;
		}else if(head.indexOf("<components") > 0) {
			// S2Container
			result = CONTAINER_SEASAR;
		}else if(head.indexOf("<beans") > 0) {
			// Spring
			result = CONTAINER_SPRING;
		}else {
			throw new DIContainerNotSupportedException();
		}
		return result;
	}
}
